# -*- coding: utf-8 -*-

from django.contrib.contenttypes.models import ContentType
from m3_django_compat import get_model

from educommon.django.db.observer import ModelObserverBase
from educommon.django.db.observer import OriginalObjectMixin
from educommon.django.db.observer import ModelOnlyObserverMixin

from . import models


class ContingentFieldsObserver(
    ModelOnlyObserverMixin,
    OriginalObjectMixin,
    ModelObserverBase
):

    u"""
    Наблюдает за изменениями полей в моделях.

    :attr dict model_fields описание моделей и их полей за которыми
    нужно следить.
    model_fields = {
        ('person', 'Person'): ('surname',)
    }

    Фиксирует факт изменения в модели ContingentModelChanged.
    """

    model_fields = {}

    def __init__(self):
        self.check_model_fields()
        super(ContingentFieldsObserver, self).__init__()

    @classmethod
    def check_model_fields(cls):
        u"""Проверяет наличие указаных полей model_fields в моделях."""
        for (app_label, model_name), fields in cls.model_fields.iteritems():
            model = get_model(app_label, model_name)
            for field in (fields(model) if callable(fields) else fields):
                model._meta.get_field(field)

    def post_save(self, context, instance, sender, *args, **kwargs):
        app_model_name = (sender._meta.app_label, sender.__name__)
        changed = False

        if context.original:
            fields = self.model_fields.get(app_model_name, [])
            for field in (fields(sender) if callable(fields) else fields):
                old_value = getattr(context.original, field)
                new_value = getattr(instance, field)
                if old_value != new_value:
                    changed = True
                    break
        else:
            changed = True

        if changed:
            models.ContingentModelChanged.objects.get_or_create(
                content_type=ContentType.objects.get_for_model(instance),
                object_id=instance.pk
            )
